#define F_CPU   12000000L    /* evaluation board runs on 4MHz */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "usbdrv.h"
#include "oddebug.h"

/* ----------------------- hardware I/O abstraction ------------------------ */

/* pin assignments:
APINS = X

CPINS = Y

PINB0 = left, b1
PINB1 = right, b2

PD0	USB-
PD1	debug tx
PD2	USB+ (int0)

*/

static signed char key[3];
static signed char lastKey[3];
static uchar index;
static int counter;
static uchar setx0, sety0;

static void hardwareInit(void)
{
uchar	i, j;
	PORTA = 0x00;
	DDRA=0;
    PORTB = 0x00;   /* activate all pull-ups */
    DDRB = 0;       /* all pins input */
    PORTC = 0x00;   /* activate all pull-ups */
    DDRC = 0;       /* all pins input */
    PORTD = 0xfa;   /* 1111 1010 bin: activate pull-ups except on USB lines */
    DDRD = 0x07;    /* 0000 0111 bin: all pins input except USB (-> USB reset) */
	j = 0;
	
	while(--j){     /* USB Reset by device only required on Watchdog Reset */
		i = 0;
		while(--i); /* delay >10ms for USB reset */
	}
    DDRD = 0x02;    /* 0000 0010 bin: remove USB reset condition */
    /* configure timer 0 for a rate of 12M/(1024 * 256) = 45.78 Hz (~22ms) */
    TCCR0 = 5;      /* timer 0 prescaler: 1024 */
}

/* ------------------------------------------------------------------------- */


static void keyPressed(void) 
{
	key[0] = PINB & 0b0000011; //buttons
	key[1] = PINA; //X input
	key[2] = PINC; //Y input
}   

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

static uchar    reportBuffer[3];    /* buffer for HID reports */ // CHANGE from 2 to 24
static uchar    idleRate;           /* in 4 ms units */

PROGMEM char usbHidReportDescriptor[50] = { /* USB report descriptor */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x02,                    // USAGE (Mouse)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE_PAGE (Pointer)
    0xa1, 0x00,                    //   COLLECTION (Physical)
    0x05, 0x09,                    //   USAGE_PAGE (Button)
    0x19, 0x01,                    //   USAGE_MINIMUM (Button 1)
    0x29, 0x03,                    //   USAGE_MAXIMUM (Button 3)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x95, 0x03,                    //   REPORT_COUNT (3)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x05,                    //   REPORT_SIZE (5)
    0x81, 0x01,                    //   Input(Cnst)
    0x05, 0x01,                    //   USAGE_PAGE(Generic Desktop)
    0x09, 0x30,                    //   USAGE(X)
    0x09, 0x31,                    //   USAGE(Y)
    0x15, 0x81,                    //   LOGICAL_MINIMUM (-127)
    0x25, 0x7f,                    //   LOGICAL_MAXIMUM (127)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x02,                    //   REPORT_COUNT (2)
    0x81, 0x06,                    //   INPUT (Data,Var,Rel)
    0xc0,                           // END_COLLECTION
    0xc0                           // END_COLLECTION
};

static void buildReport(signed char blah[])
{	
    reportBuffer[0] = blah[0];
	if(setx0==1) //if x didn't change, send 0
		reportBuffer[1]=0;
	else
		reportBuffer[1] = blah[1];
	if(sety0==1) //if y didn't change, send 0
		reportBuffer[2] = 0;
	else
		reportBuffer[2] = blah[2];
	sety0=0;
	setx0=0;
/*DESCRIPTOR REPORT STRCTURE
BYTE in 0 = last 3 bits = buttons. BYTE1 = X, BYTE 2 = Y
*/
}

uchar	usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    usbMsgPtr = reportBuffer;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            keyPressed();
			buildReport(key);
            return sizeof(reportBuffer);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
	return 0;
}

static int compare(void) //compare key and last key values
{
	int x;
	//check X and Y and set if no change to stop drift
	if(key[1]==lastKey[1])
		setx0=1;
	else
		setx0=0;
	if(key[2]==lastKey[2])
		sety0=1;
	else 
		sety0=0;
		
	for(x =0; x<3; x++)
	{	if(key[x] !=lastKey[x])
			return 1;
	}	
	return 0;

}

/* ------------------------------------------------------------------------- */

int	main(void)
{
uchar keyDidChange = 0;
uchar   idleCounter = 0;
int z;
index=0;
counter=0;
setx0=0;
sety0=0;
	//initiazlize lastKey
	for(z = 0; z<3; z++)
	{
		lastKey[z] = 0;
		key[z]=0;
	}
	
	wdt_enable(WDTO_2S);
    hardwareInit();
	odDebugInit();
	usbInit();
	sei();
	//key[1]=50;
    DBG1(0x00, 0, 0);
	for(;;){	/* main event loop */
		wdt_reset();
		usbPoll();
        if(TIFR & (1<<TOV0)){   /* 22 ms timer*/ 
            TIFR = 1<<TOV0;
            if(idleRate != 0){
                if(idleCounter > 4){ // changing to units of 
                    idleCounter -= 5;   /* 22 ms in units of 4 ms*/ 
                }else{
                    idleCounter = idleRate;
                    keyDidChange = 1;
                }
            }
        } 
		
		 if(usbInterruptIsReady()){   
            /* use last key and not current key status in order to avoid lost
              changes in key status. */
			 keyPressed();
			if(compare()==1){ //preserve data into lastKey
				for(z= 0; z<3;z++)
					lastKey[z]=key[z];
				keyDidChange = 1;
			}
				if(keyDidChange){
					buildReport(key);
					usbSetInterrupt(reportBuffer, sizeof(reportBuffer));
				}
			keyDidChange=0;
        }
	}
	return 0;
}

/* ------------------------------------------------------------------------- */
